/*
    This file is part of Android File Transfer For Linux.
    Copyright (C) 2015-2018  Vladimir Menshakov

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License,
    or (at your option) any later version.

    This library is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this library; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#ifndef AFS_FUSE_FUSEID_H
#define	AFS_FUSE_FUSEID_H

#include <fuse_lowlevel.h>

namespace mtp { namespace fuse
{

	struct FuseId
	{
		static const FuseId Root;

		fuse_ino_t Inode; //generation here?

		explicit FuseId(fuse_ino_t inode): Inode(inode) { }

		bool operator == (const FuseId &o) const
		{ return Inode == o.Inode; }
		bool operator != (const FuseId &o) const
		{ return !((*this) == o); }
		bool operator < (const FuseId &o) const
		{ return Inode < o.Inode; }
	};

	const FuseId FuseId::Root(FUSE_ROOT_ID);


}}

#endif
