#line 10 "arg_lexer.l"
#	include <cli/Tokenizer.h>
#	define register



#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */











#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 1
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif














    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
        
        
    
    






    
    
    






/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */



/* TODO: this is always defined, so inline it */
#define yyconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define yynoreturn __attribute__((__noreturn__))
#else
#define yynoreturn
#endif


    



/* Returned upon end-of-file. */
#define YY_NULL 0



/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)





/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif









    
    








/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r









    
    
    
    



/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *



/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START



/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)



/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE args_restart(yyin ,yyscanner )



#define YY_END_OF_BUFFER_CHAR 0


/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif


/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))



#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif




#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2



    
    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    



/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        yy_size_t yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = yyg->yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )



#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )


#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;



	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via args_restart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */





/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)



/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]






void args_restart (FILE *input_file ,yyscan_t yyscanner );
void args__switch_to_buffer (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
YY_BUFFER_STATE args__create_buffer (FILE *file,int size ,yyscan_t yyscanner );
void args__delete_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void args__flush_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void args_push_buffer_state (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
void args_pop_buffer_state (yyscan_t yyscanner );


static void args_ensure_buffer_stack (yyscan_t yyscanner );
static void args__load_buffer_state (yyscan_t yyscanner );
static void args__init_buffer (YY_BUFFER_STATE b,FILE *file ,yyscan_t yyscanner );



#define YY_FLUSH_BUFFER args__flush_buffer(YY_CURRENT_BUFFER ,yyscanner)


YY_BUFFER_STATE args__scan_buffer (char *base,yy_size_t size ,yyscan_t yyscanner );
YY_BUFFER_STATE args__scan_string (yyconst char *yy_str ,yyscan_t yyscanner );
YY_BUFFER_STATE args__scan_bytes (yyconst char *bytes,int len ,yyscan_t yyscanner );


void *args_alloc (yy_size_t ,yyscan_t yyscanner );
void *args_realloc (void *,yy_size_t ,yyscan_t yyscanner );
void args_free (void * ,yyscan_t yyscanner );


#define yy_new_buffer args__create_buffer



#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        args_ensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            args__create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}



#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        args_ensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            args__create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}



#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)


/* Begin user sect3 */

#define args_wrap(yyscanner) (/*CONSTCOND*/1)
#define YY_SKIP_YYWRAP

typedef unsigned char YY_CHAR;




typedef yyconst struct yy_trans_info *yy_state_type;

#define yytext_ptr yytext_r






static yy_state_type yy_get_previous_state (yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  ,yyscan_t yyscanner);
static int yy_get_next_buffer (yyscan_t yyscanner );
static void yynoreturn yy_fatal_error (yyconst char* msg ,yyscan_t yyscanner );




/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yyg->yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yyg->yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yyg->yy_c_buf_p = yy_cp;



#define YY_NUM_RULES 13
#define YY_END_OF_BUFFER 14
struct yy_trans_info
	{
	flex_int16_t yy_verify;
	flex_int16_t yy_nxt;
	};
static yyconst struct yy_trans_info yy_transition[2399] =
    {
 {   0,   0 }, {   0,2143 }, {   0,   0 }, {   0,2141 }, {   1,1032 },
 {   2,1032 }, {   3,1032 }, {   4,1032 }, {   5,1032 }, {   6,1032 },
 {   7,1032 }, {   8,1032 }, {   9,1034 }, {  10,1039 }, {  11,1034 },
 {  12,1034 }, {  13,1034 }, {  14,1032 }, {  15,1032 }, {  16,1032 },
 {  17,1032 }, {  18,1032 }, {  19,1032 }, {  20,1032 }, {  21,1032 },
 {  22,1032 }, {  23,1032 }, {  24,1032 }, {  25,1032 }, {  26,1032 },
 {  27,1032 }, {  28,1032 }, {  29,1032 }, {  30,1032 }, {  31,1032 },
 {  32,1034 }, {  33,1032 }, {  34,1036 }, {  35,1032 }, {  36,1032 },
 {  37,1032 }, {  38,1032 }, {  39,1032 }, {  40,1032 }, {  41,1032 },
 {  42,1032 }, {  43,1032 }, {  44,1032 }, {  45,1032 }, {  46,1032 },

 {  47,1032 }, {  48,1032 }, {  49,1032 }, {  50,1032 }, {  51,1032 },
 {  52,1032 }, {  53,1032 }, {  54,1032 }, {  55,1032 }, {  56,1032 },
 {  57,1032 }, {  58,1032 }, {  59,1032 }, {  60,1032 }, {  61,1032 },
 {  62,1032 }, {  63,1032 }, {  64,1032 }, {  65,1032 }, {  66,1032 },
 {  67,1032 }, {  68,1032 }, {  69,1032 }, {  70,1032 }, {  71,1032 },
 {  72,1032 }, {  73,1032 }, {  74,1032 }, {  75,1032 }, {  76,1032 },
 {  77,1032 }, {  78,1032 }, {  79,1032 }, {  80,1032 }, {  81,1032 },
 {  82,1032 }, {  83,1032 }, {  84,1032 }, {  85,1032 }, {  86,1032 },
 {  87,1032 }, {  88,1032 }, {  89,1032 }, {  90,1032 }, {  91,1032 },
 {  92,1041 }, {  93,1032 }, {  94,1032 }, {  95,1032 }, {  96,1032 },

 {  97,1032 }, {  98,1032 }, {  99,1032 }, { 100,1032 }, { 101,1032 },
 { 102,1032 }, { 103,1032 }, { 104,1032 }, { 105,1032 }, { 106,1032 },
 { 107,1032 }, { 108,1032 }, { 109,1032 }, { 110,1032 }, { 111,1032 },
 { 112,1032 }, { 113,1032 }, { 114,1032 }, { 115,1032 }, { 116,1032 },
 { 117,1032 }, { 118,1032 }, { 119,1032 }, { 120,1032 }, { 121,1032 },
 { 122,1032 }, { 123,1032 }, { 124,1032 }, { 125,1032 }, { 126,1032 },
 { 127,1032 }, { 128,1032 }, { 129,1032 }, { 130,1032 }, { 131,1032 },
 { 132,1032 }, { 133,1032 }, { 134,1032 }, { 135,1032 }, { 136,1032 },
 { 137,1032 }, { 138,1032 }, { 139,1032 }, { 140,1032 }, { 141,1032 },
 { 142,1032 }, { 143,1032 }, { 144,1032 }, { 145,1032 }, { 146,1032 },

 { 147,1032 }, { 148,1032 }, { 149,1032 }, { 150,1032 }, { 151,1032 },
 { 152,1032 }, { 153,1032 }, { 154,1032 }, { 155,1032 }, { 156,1032 },
 { 157,1032 }, { 158,1032 }, { 159,1032 }, { 160,1032 }, { 161,1032 },
 { 162,1032 }, { 163,1032 }, { 164,1032 }, { 165,1032 }, { 166,1032 },
 { 167,1032 }, { 168,1032 }, { 169,1032 }, { 170,1032 }, { 171,1032 },
 { 172,1032 }, { 173,1032 }, { 174,1032 }, { 175,1032 }, { 176,1032 },
 { 177,1032 }, { 178,1032 }, { 179,1032 }, { 180,1032 }, { 181,1032 },
 { 182,1032 }, { 183,1032 }, { 184,1032 }, { 185,1032 }, { 186,1032 },
 { 187,1032 }, { 188,1032 }, { 189,1032 }, { 190,1032 }, { 191,1032 },
 { 192,1032 }, { 193,1032 }, { 194,1032 }, { 195,1032 }, { 196,1032 },

 { 197,1032 }, { 198,1032 }, { 199,1032 }, { 200,1032 }, { 201,1032 },
 { 202,1032 }, { 203,1032 }, { 204,1032 }, { 205,1032 }, { 206,1032 },
 { 207,1032 }, { 208,1032 }, { 209,1032 }, { 210,1032 }, { 211,1032 },
 { 212,1032 }, { 213,1032 }, { 214,1032 }, { 215,1032 }, { 216,1032 },
 { 217,1032 }, { 218,1032 }, { 219,1032 }, { 220,1032 }, { 221,1032 },
 { 222,1032 }, { 223,1032 }, { 224,1032 }, { 225,1032 }, { 226,1032 },
 { 227,1032 }, { 228,1032 }, { 229,1032 }, { 230,1032 }, { 231,1032 },
 { 232,1032 }, { 233,1032 }, { 234,1032 }, { 235,1032 }, { 236,1032 },
 { 237,1032 }, { 238,1032 }, { 239,1032 }, { 240,1032 }, { 241,1032 },
 { 242,1032 }, { 243,1032 }, { 244,1032 }, { 245,1032 }, { 246,1032 },

 { 247,1032 }, { 248,1032 }, { 249,1032 }, { 250,1032 }, { 251,1032 },
 { 252,1032 }, { 253,1032 }, { 254,1032 }, { 255,1032 }, { 256,1032 },
 {   0,   0 }, {   0,1883 }, {   1, 774 }, {   2, 774 }, {   3, 774 },
 {   4, 774 }, {   5, 774 }, {   6, 774 }, {   7, 774 }, {   8, 774 },
 {   9, 776 }, {  10, 781 }, {  11, 776 }, {  12, 776 }, {  13, 776 },
 {  14, 774 }, {  15, 774 }, {  16, 774 }, {  17, 774 }, {  18, 774 },
 {  19, 774 }, {  20, 774 }, {  21, 774 }, {  22, 774 }, {  23, 774 },
 {  24, 774 }, {  25, 774 }, {  26, 774 }, {  27, 774 }, {  28, 774 },
 {  29, 774 }, {  30, 774 }, {  31, 774 }, {  32, 776 }, {  33, 774 },
 {  34, 778 }, {  35, 774 }, {  36, 774 }, {  37, 774 }, {  38, 774 },

 {  39, 774 }, {  40, 774 }, {  41, 774 }, {  42, 774 }, {  43, 774 },
 {  44, 774 }, {  45, 774 }, {  46, 774 }, {  47, 774 }, {  48, 774 },
 {  49, 774 }, {  50, 774 }, {  51, 774 }, {  52, 774 }, {  53, 774 },
 {  54, 774 }, {  55, 774 }, {  56, 774 }, {  57, 774 }, {  58, 774 },
 {  59, 774 }, {  60, 774 }, {  61, 774 }, {  62, 774 }, {  63, 774 },
 {  64, 774 }, {  65, 774 }, {  66, 774 }, {  67, 774 }, {  68, 774 },
 {  69, 774 }, {  70, 774 }, {  71, 774 }, {  72, 774 }, {  73, 774 },
 {  74, 774 }, {  75, 774 }, {  76, 774 }, {  77, 774 }, {  78, 774 },
 {  79, 774 }, {  80, 774 }, {  81, 774 }, {  82, 774 }, {  83, 774 },
 {  84, 774 }, {  85, 774 }, {  86, 774 }, {  87, 774 }, {  88, 774 },

 {  89, 774 }, {  90, 774 }, {  91, 774 }, {  92, 783 }, {  93, 774 },
 {  94, 774 }, {  95, 774 }, {  96, 774 }, {  97, 774 }, {  98, 774 },
 {  99, 774 }, { 100, 774 }, { 101, 774 }, { 102, 774 }, { 103, 774 },
 { 104, 774 }, { 105, 774 }, { 106, 774 }, { 107, 774 }, { 108, 774 },
 { 109, 774 }, { 110, 774 }, { 111, 774 }, { 112, 774 }, { 113, 774 },
 { 114, 774 }, { 115, 774 }, { 116, 774 }, { 117, 774 }, { 118, 774 },
 { 119, 774 }, { 120, 774 }, { 121, 774 }, { 122, 774 }, { 123, 774 },
 { 124, 774 }, { 125, 774 }, { 126, 774 }, { 127, 774 }, { 128, 774 },
 { 129, 774 }, { 130, 774 }, { 131, 774 }, { 132, 774 }, { 133, 774 },
 { 134, 774 }, { 135, 774 }, { 136, 774 }, { 137, 774 }, { 138, 774 },

 { 139, 774 }, { 140, 774 }, { 141, 774 }, { 142, 774 }, { 143, 774 },
 { 144, 774 }, { 145, 774 }, { 146, 774 }, { 147, 774 }, { 148, 774 },
 { 149, 774 }, { 150, 774 }, { 151, 774 }, { 152, 774 }, { 153, 774 },
 { 154, 774 }, { 155, 774 }, { 156, 774 }, { 157, 774 }, { 158, 774 },
 { 159, 774 }, { 160, 774 }, { 161, 774 }, { 162, 774 }, { 163, 774 },
 { 164, 774 }, { 165, 774 }, { 166, 774 }, { 167, 774 }, { 168, 774 },
 { 169, 774 }, { 170, 774 }, { 171, 774 }, { 172, 774 }, { 173, 774 },
 { 174, 774 }, { 175, 774 }, { 176, 774 }, { 177, 774 }, { 178, 774 },
 { 179, 774 }, { 180, 774 }, { 181, 774 }, { 182, 774 }, { 183, 774 },
 { 184, 774 }, { 185, 774 }, { 186, 774 }, { 187, 774 }, { 188, 774 },

 { 189, 774 }, { 190, 774 }, { 191, 774 }, { 192, 774 }, { 193, 774 },
 { 194, 774 }, { 195, 774 }, { 196, 774 }, { 197, 774 }, { 198, 774 },
 { 199, 774 }, { 200, 774 }, { 201, 774 }, { 202, 774 }, { 203, 774 },
 { 204, 774 }, { 205, 774 }, { 206, 774 }, { 207, 774 }, { 208, 774 },
 { 209, 774 }, { 210, 774 }, { 211, 774 }, { 212, 774 }, { 213, 774 },
 { 214, 774 }, { 215, 774 }, { 216, 774 }, { 217, 774 }, { 218, 774 },
 { 219, 774 }, { 220, 774 }, { 221, 774 }, { 222, 774 }, { 223, 774 },
 { 224, 774 }, { 225, 774 }, { 226, 774 }, { 227, 774 }, { 228, 774 },
 { 229, 774 }, { 230, 774 }, { 231, 774 }, { 232, 774 }, { 233, 774 },
 { 234, 774 }, { 235, 774 }, { 236, 774 }, { 237, 774 }, { 238, 774 },

 { 239, 774 }, { 240, 774 }, { 241, 774 }, { 242, 774 }, { 243, 774 },
 { 244, 774 }, { 245, 774 }, { 246, 774 }, { 247, 774 }, { 248, 774 },
 { 249, 774 }, { 250, 774 }, { 251, 774 }, { 252, 774 }, { 253, 774 },
 { 254, 774 }, { 255, 774 }, { 256, 774 }, {   0,   0 }, {   0,1625 },
 {   1, 559 }, {   2, 559 }, {   3, 559 }, {   4, 559 }, {   5, 559 },
 {   6, 559 }, {   7, 559 }, {   8, 559 }, {   9, 559 }, {  10, 817 },
 {  11, 559 }, {  12, 559 }, {  13, 559 }, {  14, 559 }, {  15, 559 },
 {  16, 559 }, {  17, 559 }, {  18, 559 }, {  19, 559 }, {  20, 559 },
 {  21, 559 }, {  22, 559 }, {  23, 559 }, {  24, 559 }, {  25, 559 },
 {  26, 559 }, {  27, 559 }, {  28, 559 }, {  29, 559 }, {  30, 559 },

 {  31, 559 }, {  32, 559 }, {  33, 559 }, {  34, 538 }, {  35, 559 },
 {  36, 559 }, {  37, 559 }, {  38, 559 }, {  39, 559 }, {  40, 559 },
 {  41, 559 }, {  42, 559 }, {  43, 559 }, {  44, 559 }, {  45, 559 },
 {  46, 559 }, {  47, 559 }, {  48, 559 }, {  49, 559 }, {  50, 559 },
 {  51, 559 }, {  52, 559 }, {  53, 559 }, {  54, 559 }, {  55, 559 },
 {  56, 559 }, {  57, 559 }, {  58, 559 }, {  59, 559 }, {  60, 559 },
 {  61, 559 }, {  62, 559 }, {  63, 559 }, {  64, 559 }, {  65, 559 },
 {  66, 559 }, {  67, 559 }, {  68, 559 }, {  69, 559 }, {  70, 559 },
 {  71, 559 }, {  72, 559 }, {  73, 559 }, {  74, 559 }, {  75, 559 },
 {  76, 559 }, {  77, 559 }, {  78, 559 }, {  79, 559 }, {  80, 559 },

 {  81, 559 }, {  82, 559 }, {  83, 559 }, {  84, 559 }, {  85, 559 },
 {  86, 559 }, {  87, 559 }, {  88, 559 }, {  89, 559 }, {  90, 559 },
 {  91, 559 }, {  92,1075 }, {  93, 559 }, {  94, 559 }, {  95, 559 },
 {  96, 559 }, {  97, 559 }, {  98, 559 }, {  99, 559 }, { 100, 559 },
 { 101, 559 }, { 102, 559 }, { 103, 559 }, { 104, 559 }, { 105, 559 },
 { 106, 559 }, { 107, 559 }, { 108, 559 }, { 109, 559 }, { 110, 559 },
 { 111, 559 }, { 112, 559 }, { 113, 559 }, { 114, 559 }, { 115, 559 },
 { 116, 559 }, { 117, 559 }, { 118, 559 }, { 119, 559 }, { 120, 559 },
 { 121, 559 }, { 122, 559 }, { 123, 559 }, { 124, 559 }, { 125, 559 },
 { 126, 559 }, { 127, 559 }, { 128, 559 }, { 129, 559 }, { 130, 559 },

 { 131, 559 }, { 132, 559 }, { 133, 559 }, { 134, 559 }, { 135, 559 },
 { 136, 559 }, { 137, 559 }, { 138, 559 }, { 139, 559 }, { 140, 559 },
 { 141, 559 }, { 142, 559 }, { 143, 559 }, { 144, 559 }, { 145, 559 },
 { 146, 559 }, { 147, 559 }, { 148, 559 }, { 149, 559 }, { 150, 559 },
 { 151, 559 }, { 152, 559 }, { 153, 559 }, { 154, 559 }, { 155, 559 },
 { 156, 559 }, { 157, 559 }, { 158, 559 }, { 159, 559 }, { 160, 559 },
 { 161, 559 }, { 162, 559 }, { 163, 559 }, { 164, 559 }, { 165, 559 },
 { 166, 559 }, { 167, 559 }, { 168, 559 }, { 169, 559 }, { 170, 559 },
 { 171, 559 }, { 172, 559 }, { 173, 559 }, { 174, 559 }, { 175, 559 },
 { 176, 559 }, { 177, 559 }, { 178, 559 }, { 179, 559 }, { 180, 559 },

 { 181, 559 }, { 182, 559 }, { 183, 559 }, { 184, 559 }, { 185, 559 },
 { 186, 559 }, { 187, 559 }, { 188, 559 }, { 189, 559 }, { 190, 559 },
 { 191, 559 }, { 192, 559 }, { 193, 559 }, { 194, 559 }, { 195, 559 },
 { 196, 559 }, { 197, 559 }, { 198, 559 }, { 199, 559 }, { 200, 559 },
 { 201, 559 }, { 202, 559 }, { 203, 559 }, { 204, 559 }, { 205, 559 },
 { 206, 559 }, { 207, 559 }, { 208, 559 }, { 209, 559 }, { 210, 559 },
 { 211, 559 }, { 212, 559 }, { 213, 559 }, { 214, 559 }, { 215, 559 },
 { 216, 559 }, { 217, 559 }, { 218, 559 }, { 219, 559 }, { 220, 559 },
 { 221, 559 }, { 222, 559 }, { 223, 559 }, { 224, 559 }, { 225, 559 },
 { 226, 559 }, { 227, 559 }, { 228, 559 }, { 229, 559 }, { 230, 559 },

 { 231, 559 }, { 232, 559 }, { 233, 559 }, { 234, 559 }, { 235, 559 },
 { 236, 559 }, { 237, 559 }, { 238, 559 }, { 239, 559 }, { 240, 559 },
 { 241, 559 }, { 242, 559 }, { 243, 559 }, { 244, 559 }, { 245, 559 },
 { 246, 559 }, { 247, 559 }, { 248, 559 }, { 249, 559 }, { 250, 559 },
 { 251, 559 }, { 252, 559 }, { 253, 559 }, { 254, 559 }, { 255, 559 },
 { 256, 559 }, {   0,   0 }, {   0,1367 }, {   1, 301 }, {   2, 301 },
 {   3, 301 }, {   4, 301 }, {   5, 301 }, {   6, 301 }, {   7, 301 },
 {   8, 301 }, {   9, 301 }, {  10, 559 }, {  11, 301 }, {  12, 301 },
 {  13, 301 }, {  14, 301 }, {  15, 301 }, {  16, 301 }, {  17, 301 },
 {  18, 301 }, {  19, 301 }, {  20, 301 }, {  21, 301 }, {  22, 301 },

 {  23, 301 }, {  24, 301 }, {  25, 301 }, {  26, 301 }, {  27, 301 },
 {  28, 301 }, {  29, 301 }, {  30, 301 }, {  31, 301 }, {  32, 301 },
 {  33, 301 }, {  34, 280 }, {  35, 301 }, {  36, 301 }, {  37, 301 },
 {  38, 301 }, {  39, 301 }, {  40, 301 }, {  41, 301 }, {  42, 301 },
 {  43, 301 }, {  44, 301 }, {  45, 301 }, {  46, 301 }, {  47, 301 },
 {  48, 301 }, {  49, 301 }, {  50, 301 }, {  51, 301 }, {  52, 301 },
 {  53, 301 }, {  54, 301 }, {  55, 301 }, {  56, 301 }, {  57, 301 },
 {  58, 301 }, {  59, 301 }, {  60, 301 }, {  61, 301 }, {  62, 301 },
 {  63, 301 }, {  64, 301 }, {  65, 301 }, {  66, 301 }, {  67, 301 },
 {  68, 301 }, {  69, 301 }, {  70, 301 }, {  71, 301 }, {  72, 301 },

 {  73, 301 }, {  74, 301 }, {  75, 301 }, {  76, 301 }, {  77, 301 },
 {  78, 301 }, {  79, 301 }, {  80, 301 }, {  81, 301 }, {  82, 301 },
 {  83, 301 }, {  84, 301 }, {  85, 301 }, {  86, 301 }, {  87, 301 },
 {  88, 301 }, {  89, 301 }, {  90, 301 }, {  91, 301 }, {  92, 817 },
 {  93, 301 }, {  94, 301 }, {  95, 301 }, {  96, 301 }, {  97, 301 },
 {  98, 301 }, {  99, 301 }, { 100, 301 }, { 101, 301 }, { 102, 301 },
 { 103, 301 }, { 104, 301 }, { 105, 301 }, { 106, 301 }, { 107, 301 },
 { 108, 301 }, { 109, 301 }, { 110, 301 }, { 111, 301 }, { 112, 301 },
 { 113, 301 }, { 114, 301 }, { 115, 301 }, { 116, 301 }, { 117, 301 },
 { 118, 301 }, { 119, 301 }, { 120, 301 }, { 121, 301 }, { 122, 301 },

 { 123, 301 }, { 124, 301 }, { 125, 301 }, { 126, 301 }, { 127, 301 },
 { 128, 301 }, { 129, 301 }, { 130, 301 }, { 131, 301 }, { 132, 301 },
 { 133, 301 }, { 134, 301 }, { 135, 301 }, { 136, 301 }, { 137, 301 },
 { 138, 301 }, { 139, 301 }, { 140, 301 }, { 141, 301 }, { 142, 301 },
 { 143, 301 }, { 144, 301 }, { 145, 301 }, { 146, 301 }, { 147, 301 },
 { 148, 301 }, { 149, 301 }, { 150, 301 }, { 151, 301 }, { 152, 301 },
 { 153, 301 }, { 154, 301 }, { 155, 301 }, { 156, 301 }, { 157, 301 },
 { 158, 301 }, { 159, 301 }, { 160, 301 }, { 161, 301 }, { 162, 301 },
 { 163, 301 }, { 164, 301 }, { 165, 301 }, { 166, 301 }, { 167, 301 },
 { 168, 301 }, { 169, 301 }, { 170, 301 }, { 171, 301 }, { 172, 301 },

 { 173, 301 }, { 174, 301 }, { 175, 301 }, { 176, 301 }, { 177, 301 },
 { 178, 301 }, { 179, 301 }, { 180, 301 }, { 181, 301 }, { 182, 301 },
 { 183, 301 }, { 184, 301 }, { 185, 301 }, { 186, 301 }, { 187, 301 },
 { 188, 301 }, { 189, 301 }, { 190, 301 }, { 191, 301 }, { 192, 301 },
 { 193, 301 }, { 194, 301 }, { 195, 301 }, { 196, 301 }, { 197, 301 },
 { 198, 301 }, { 199, 301 }, { 200, 301 }, { 201, 301 }, { 202, 301 },
 { 203, 301 }, { 204, 301 }, { 205, 301 }, { 206, 301 }, { 207, 301 },
 { 208, 301 }, { 209, 301 }, { 210, 301 }, { 211, 301 }, { 212, 301 },
 { 213, 301 }, { 214, 301 }, { 215, 301 }, { 216, 301 }, { 217, 301 },
 { 218, 301 }, { 219, 301 }, { 220, 301 }, { 221, 301 }, { 222, 301 },

 { 223, 301 }, { 224, 301 }, { 225, 301 }, { 226, 301 }, { 227, 301 },
 { 228, 301 }, { 229, 301 }, { 230, 301 }, { 231, 301 }, { 232, 301 },
 { 233, 301 }, { 234, 301 }, { 235, 301 }, { 236, 301 }, { 237, 301 },
 { 238, 301 }, { 239, 301 }, { 240, 301 }, { 241, 301 }, { 242, 301 },
 { 243, 301 }, { 244, 301 }, { 245, 301 }, { 246, 301 }, { 247, 301 },
 { 248, 301 }, { 249, 301 }, { 250, 301 }, { 251, 301 }, { 252, 301 },
 { 253, 301 }, { 254, 301 }, { 255, 301 }, { 256, 301 }, {   0,  12 },
 {   0,1109 }, {   0,  11 }, {   0,1107 }, {   0,   1 }, {   0,1105 },
 {   0,   0 }, {   0,  11 }, {   0,1102 }, {   0,  12 }, {   0,1100 },
 {   0,   0 }, {   9, 815 }, {  10, 815 }, {  11, 815 }, {  12, 815 },

 {  13, 815 }, {   9, 810 }, {  10, 810 }, {  11, 810 }, {  12, 810 },
 {  13, 810 }, {   0,   8 }, {   0,1087 }, {   0,  10 }, {   0,1085 },
 {   0,   7 }, {   0,1083 }, {   0,   6 }, {   0,1081 }, {   0,   4 },
 {   0,1079 }, {   0,   2 }, {   0,1077 }, {   0,   0 }, {  32, 815 },
 {   0,   5 }, {   0,1073 }, {   0,   3 }, {   0,1071 }, {  32, 810 },
 {   0,   0 }, {  32,  15 }, {   0,   9 }, {   0,1066 }, {   1, 808 },
 {   2, 808 }, {   3, 808 }, {   4, 808 }, {   5, 808 }, {   6, 808 },
 {   7, 808 }, {   8, 808 }, {   9, 808 }, {  10, 808 }, {  11, 808 },
 {  12, 808 }, {  13, 808 }, {  14, 808 }, {  15, 808 }, {  16, 808 },
 {  17, 808 }, {  18, 808 }, {  19, 808 }, {  20, 808 }, {  21, 808 },

 {  22, 808 }, {  23, 808 }, {  24, 808 }, {  25, 808 }, {  26, 808 },
 {  27, 808 }, {  28, 808 }, {  29, 808 }, {  30, 808 }, {  31, 808 },
 {  32, 808 }, {  33, 808 }, {   0,   0 }, {  35, 808 }, {  36, 808 },
 {  37, 808 }, {  38, 808 }, {  39, 808 }, {  40, 808 }, {  41, 808 },
 {  42, 808 }, {  43, 808 }, {  44, 808 }, {  45, 808 }, {  46, 808 },
 {  47, 808 }, {  48, 808 }, {  49, 808 }, {  50, 808 }, {  51, 808 },
 {  52, 808 }, {  53, 808 }, {  54, 808 }, {  55, 808 }, {  56, 808 },
 {  57, 808 }, {  58, 808 }, {  59, 808 }, {  60, 808 }, {  61, 808 },
 {  62, 808 }, {  63, 808 }, {  64, 808 }, {  65, 808 }, {  66, 808 },
 {  67, 808 }, {  68, 808 }, {  69, 808 }, {  70, 808 }, {  71, 808 },

 {  72, 808 }, {  73, 808 }, {  74, 808 }, {  75, 808 }, {  76, 808 },
 {  77, 808 }, {  78, 808 }, {  79, 808 }, {  80, 808 }, {  81, 808 },
 {  82, 808 }, {  83, 808 }, {  84, 808 }, {  85, 808 }, {  86, 808 },
 {  87, 808 }, {  88, 808 }, {  89, 808 }, {  90, 808 }, {  91, 808 },
 {   0,   0 }, {  93, 808 }, {  94, 808 }, {  95, 808 }, {  96, 808 },
 {  97, 808 }, {  98, 808 }, {  99, 808 }, { 100, 808 }, { 101, 808 },
 { 102, 808 }, { 103, 808 }, { 104, 808 }, { 105, 808 }, { 106, 808 },
 { 107, 808 }, { 108, 808 }, { 109, 808 }, { 110, 808 }, { 111, 808 },
 { 112, 808 }, { 113, 808 }, { 114, 808 }, { 115, 808 }, { 116, 808 },
 { 117, 808 }, { 118, 808 }, { 119, 808 }, { 120, 808 }, { 121, 808 },

 { 122, 808 }, { 123, 808 }, { 124, 808 }, { 125, 808 }, { 126, 808 },
 { 127, 808 }, { 128, 808 }, { 129, 808 }, { 130, 808 }, { 131, 808 },
 { 132, 808 }, { 133, 808 }, { 134, 808 }, { 135, 808 }, { 136, 808 },
 { 137, 808 }, { 138, 808 }, { 139, 808 }, { 140, 808 }, { 141, 808 },
 { 142, 808 }, { 143, 808 }, { 144, 808 }, { 145, 808 }, { 146, 808 },
 { 147, 808 }, { 148, 808 }, { 149, 808 }, { 150, 808 }, { 151, 808 },
 { 152, 808 }, { 153, 808 }, { 154, 808 }, { 155, 808 }, { 156, 808 },
 { 157, 808 }, { 158, 808 }, { 159, 808 }, { 160, 808 }, { 161, 808 },
 { 162, 808 }, { 163, 808 }, { 164, 808 }, { 165, 808 }, { 166, 808 },
 { 167, 808 }, { 168, 808 }, { 169, 808 }, { 170, 808 }, { 171, 808 },

 { 172, 808 }, { 173, 808 }, { 174, 808 }, { 175, 808 }, { 176, 808 },
 { 177, 808 }, { 178, 808 }, { 179, 808 }, { 180, 808 }, { 181, 808 },
 { 182, 808 }, { 183, 808 }, { 184, 808 }, { 185, 808 }, { 186, 808 },
 { 187, 808 }, { 188, 808 }, { 189, 808 }, { 190, 808 }, { 191, 808 },
 { 192, 808 }, { 193, 808 }, { 194, 808 }, { 195, 808 }, { 196, 808 },
 { 197, 808 }, { 198, 808 }, { 199, 808 }, { 200, 808 }, { 201, 808 },
 { 202, 808 }, { 203, 808 }, { 204, 808 }, { 205, 808 }, { 206, 808 },
 { 207, 808 }, { 208, 808 }, { 209, 808 }, { 210, 808 }, { 211, 808 },
 { 212, 808 }, { 213, 808 }, { 214, 808 }, { 215, 808 }, { 216, 808 },
 { 217, 808 }, { 218, 808 }, { 219, 808 }, { 220, 808 }, { 221, 808 },

 { 222, 808 }, { 223, 808 }, { 224, 808 }, { 225, 808 }, { 226, 808 },
 { 227, 808 }, { 228, 808 }, { 229, 808 }, { 230, 808 }, { 231, 808 },
 { 232, 808 }, { 233, 808 }, { 234, 808 }, { 235, 808 }, { 236, 808 },
 { 237, 808 }, { 238, 808 }, { 239, 808 }, { 240, 808 }, { 241, 808 },
 { 242, 808 }, { 243, 808 }, { 244, 808 }, { 245, 808 }, { 246, 808 },
 { 247, 808 }, { 248, 808 }, { 249, 808 }, { 250, 808 }, { 251, 808 },
 { 252, 808 }, { 253, 808 }, { 254, 808 }, { 255, 808 }, { 256, 808 },
 {   0,   9 }, {   0, 808 }, {   1, 550 }, {   2, 550 }, {   3, 550 },
 {   4, 550 }, {   5, 550 }, {   6, 550 }, {   7, 550 }, {   8, 550 },
 {   9, 550 }, {  10, 550 }, {  11, 550 }, {  12, 550 }, {  13, 550 },

 {  14, 550 }, {  15, 550 }, {  16, 550 }, {  17, 550 }, {  18, 550 },
 {  19, 550 }, {  20, 550 }, {  21, 550 }, {  22, 550 }, {  23, 550 },
 {  24, 550 }, {  25, 550 }, {  26, 550 }, {  27, 550 }, {  28, 550 },
 {  29, 550 }, {  30, 550 }, {  31, 550 }, {  32, 550 }, {  33, 550 },
 {   0,   0 }, {  35, 550 }, {  36, 550 }, {  37, 550 }, {  38, 550 },
 {  39, 550 }, {  40, 550 }, {  41, 550 }, {  42, 550 }, {  43, 550 },
 {  44, 550 }, {  45, 550 }, {  46, 550 }, {  47, 550 }, {  48, 550 },
 {  49, 550 }, {  50, 550 }, {  51, 550 }, {  52, 550 }, {  53, 550 },
 {  54, 550 }, {  55, 550 }, {  56, 550 }, {  57, 550 }, {  58, 550 },
 {  59, 550 }, {  60, 550 }, {  61, 550 }, {  62, 550 }, {  63, 550 },

 {  64, 550 }, {  65, 550 }, {  66, 550 }, {  67, 550 }, {  68, 550 },
 {  69, 550 }, {  70, 550 }, {  71, 550 }, {  72, 550 }, {  73, 550 },
 {  74, 550 }, {  75, 550 }, {  76, 550 }, {  77, 550 }, {  78, 550 },
 {  79, 550 }, {  80, 550 }, {  81, 550 }, {  82, 550 }, {  83, 550 },
 {  84, 550 }, {  85, 550 }, {  86, 550 }, {  87, 550 }, {  88, 550 },
 {  89, 550 }, {  90, 550 }, {  91, 550 }, {   0,   0 }, {  93, 550 },
 {  94, 550 }, {  95, 550 }, {  96, 550 }, {  97, 550 }, {  98, 550 },
 {  99, 550 }, { 100, 550 }, { 101, 550 }, { 102, 550 }, { 103, 550 },
 { 104, 550 }, { 105, 550 }, { 106, 550 }, { 107, 550 }, { 108, 550 },
 { 109, 550 }, { 110, 550 }, { 111, 550 }, { 112, 550 }, { 113, 550 },

 { 114, 550 }, { 115, 550 }, { 116, 550 }, { 117, 550 }, { 118, 550 },
 { 119, 550 }, { 120, 550 }, { 121, 550 }, { 122, 550 }, { 123, 550 },
 { 124, 550 }, { 125, 550 }, { 126, 550 }, { 127, 550 }, { 128, 550 },
 { 129, 550 }, { 130, 550 }, { 131, 550 }, { 132, 550 }, { 133, 550 },
 { 134, 550 }, { 135, 550 }, { 136, 550 }, { 137, 550 }, { 138, 550 },
 { 139, 550 }, { 140, 550 }, { 141, 550 }, { 142, 550 }, { 143, 550 },
 { 144, 550 }, { 145, 550 }, { 146, 550 }, { 147, 550 }, { 148, 550 },
 { 149, 550 }, { 150, 550 }, { 151, 550 }, { 152, 550 }, { 153, 550 },
 { 154, 550 }, { 155, 550 }, { 156, 550 }, { 157, 550 }, { 158, 550 },
 { 159, 550 }, { 160, 550 }, { 161, 550 }, { 162, 550 }, { 163, 550 },

 { 164, 550 }, { 165, 550 }, { 166, 550 }, { 167, 550 }, { 168, 550 },
 { 169, 550 }, { 170, 550 }, { 171, 550 }, { 172, 550 }, { 173, 550 },
 { 174, 550 }, { 175, 550 }, { 176, 550 }, { 177, 550 }, { 178, 550 },
 { 179, 550 }, { 180, 550 }, { 181, 550 }, { 182, 550 }, { 183, 550 },
 { 184, 550 }, { 185, 550 }, { 186, 550 }, { 187, 550 }, { 188, 550 },
 { 189, 550 }, { 190, 550 }, { 191, 550 }, { 192, 550 }, { 193, 550 },
 { 194, 550 }, { 195, 550 }, { 196, 550 }, { 197, 550 }, { 198, 550 },
 { 199, 550 }, { 200, 550 }, { 201, 550 }, { 202, 550 }, { 203, 550 },
 { 204, 550 }, { 205, 550 }, { 206, 550 }, { 207, 550 }, { 208, 550 },
 { 209, 550 }, { 210, 550 }, { 211, 550 }, { 212, 550 }, { 213, 550 },

 { 214, 550 }, { 215, 550 }, { 216, 550 }, { 217, 550 }, { 218, 550 },
 { 219, 550 }, { 220, 550 }, { 221, 550 }, { 222, 550 }, { 223, 550 },
 { 224, 550 }, { 225, 550 }, { 226, 550 }, { 227, 550 }, { 228, 550 },
 { 229, 550 }, { 230, 550 }, { 231, 550 }, { 232, 550 }, { 233, 550 },
 { 234, 550 }, { 235, 550 }, { 236, 550 }, { 237, 550 }, { 238, 550 },
 { 239, 550 }, { 240, 550 }, { 241, 550 }, { 242, 550 }, { 243, 550 },
 { 244, 550 }, { 245, 550 }, { 246, 550 }, { 247, 550 }, { 248, 550 },
 { 249, 550 }, { 250, 550 }, { 251, 550 }, { 252, 550 }, { 253, 550 },
 { 254, 550 }, { 255, 550 }, { 256, 550 }, {   0,  12 }, {   0, 550 },
 {   1,-533 }, {   2,-533 }, {   3,-533 }, {   4,-533 }, {   5,-533 },

 {   6,-533 }, {   7,-533 }, {   8,-533 }, {   9,-533 }, {   0,   0 },
 {  11,-533 }, {  12,-533 }, {  13,-533 }, {  14,-533 }, {  15,-533 },
 {  16,-533 }, {  17,-533 }, {  18,-533 }, {  19,-533 }, {  20,-533 },
 {  21,-533 }, {  22,-533 }, {  23,-533 }, {  24,-533 }, {  25,-533 },
 {  26,-533 }, {  27,-533 }, {  28,-533 }, {  29,-533 }, {  30,-533 },
 {  31,-533 }, {  32,-533 }, {  33,-533 }, {  34,-533 }, {  35,-533 },
 {  36,-533 }, {  37,-533 }, {  38,-533 }, {  39,-533 }, {  40,-533 },
 {  41,-533 }, {  42,-533 }, {  43,-533 }, {  44,-533 }, {  45,-533 },
 {  46,-533 }, {  47,-533 }, {  48,-533 }, {  49,-533 }, {  50,-533 },
 {  51,-533 }, {  52,-533 }, {  53,-533 }, {  54,-533 }, {  55,-533 },

 {  56,-533 }, {  57,-533 }, {  58,-533 }, {  59,-533 }, {  60,-533 },
 {  61,-533 }, {  62,-533 }, {  63,-533 }, {  64,-533 }, {  65,-533 },
 {  66,-533 }, {  67,-533 }, {  68,-533 }, {  69,-533 }, {  70,-533 },
 {  71,-533 }, {  72,-533 }, {  73,-533 }, {  74,-533 }, {  75,-533 },
 {  76,-533 }, {  77,-533 }, {  78,-533 }, {  79,-533 }, {  80,-533 },
 {  81,-533 }, {  82,-533 }, {  83,-533 }, {  84,-533 }, {  85,-533 },
 {  86,-533 }, {  87,-533 }, {  88,-533 }, {  89,-533 }, {  90,-533 },
 {  91,-533 }, {  92,-533 }, {  93,-533 }, {  94,-533 }, {  95,-533 },
 {  96,-533 }, {  97,-533 }, {  98,-531 }, {  99,-533 }, { 100,-533 },
 { 101,-533 }, { 102,-529 }, { 103,-533 }, { 104,-533 }, { 105,-533 },

 { 106,-533 }, { 107,-533 }, { 108,-533 }, { 109,-533 }, { 110,-527 },
 { 111,-533 }, { 112,-533 }, { 113,-533 }, { 114,-523 }, { 115,-533 },
 { 116,-521 }, { 117,-533 }, { 118,-533 }, { 119,-533 }, { 120,-533 },
 { 121,-533 }, { 122,-533 }, { 123,-533 }, { 124,-533 }, { 125,-533 },
 { 126,-533 }, { 127,-533 }, { 128,-533 }, { 129,-533 }, { 130,-533 },
 { 131,-533 }, { 132,-533 }, { 133,-533 }, { 134,-533 }, { 135,-533 },
 { 136,-533 }, { 137,-533 }, { 138,-533 }, { 139,-533 }, { 140,-533 },
 { 141,-533 }, { 142,-533 }, { 143,-533 }, { 144,-533 }, { 145,-533 },
 { 146,-533 }, { 147,-533 }, { 148,-533 }, { 149,-533 }, { 150,-533 },
 { 151,-533 }, { 152,-533 }, { 153,-533 }, { 154,-533 }, { 155,-533 },

 { 156,-533 }, { 157,-533 }, { 158,-533 }, { 159,-533 }, { 160,-533 },
 { 161,-533 }, { 162,-533 }, { 163,-533 }, { 164,-533 }, { 165,-533 },
 { 166,-533 }, { 167,-533 }, { 168,-533 }, { 169,-533 }, { 170,-533 },
 { 171,-533 }, { 172,-533 }, { 173,-533 }, { 174,-533 }, { 175,-533 },
 { 176,-533 }, { 177,-533 }, { 178,-533 }, { 179,-533 }, { 180,-533 },
 { 181,-533 }, { 182,-533 }, { 183,-533 }, { 184,-533 }, { 185,-533 },
 { 186,-533 }, { 187,-533 }, { 188,-533 }, { 189,-533 }, { 190,-533 },
 { 191,-533 }, { 192,-533 }, { 193,-533 }, { 194,-533 }, { 195,-533 },
 { 196,-533 }, { 197,-533 }, { 198,-533 }, { 199,-533 }, { 200,-533 },
 { 201,-533 }, { 202,-533 }, { 203,-533 }, { 204,-533 }, { 205,-533 },

 { 206,-533 }, { 207,-533 }, { 208,-533 }, { 209,-533 }, { 210,-533 },
 { 211,-533 }, { 212,-533 }, { 213,-533 }, { 214,-533 }, { 215,-533 },
 { 216,-533 }, { 217,-533 }, { 218,-533 }, { 219,-533 }, { 220,-533 },
 { 221,-533 }, { 222,-533 }, { 223,-533 }, { 224,-533 }, { 225,-533 },
 { 226,-533 }, { 227,-533 }, { 228,-533 }, { 229,-533 }, { 230,-533 },
 { 231,-533 }, { 232,-533 }, { 233,-533 }, { 234,-533 }, { 235,-533 },
 { 236,-533 }, { 237,-533 }, { 238,-533 }, { 239,-533 }, { 240,-533 },
 { 241,-533 }, { 242,-533 }, { 243,-533 }, { 244,-533 }, { 245,-533 },
 { 246,-533 }, { 247,-533 }, { 248,-533 }, { 249,-533 }, { 250,-533 },
 { 251,-533 }, { 252,-533 }, { 253,-533 }, { 254,-533 }, { 255,-533 },

 { 256,-533 }, {   0,  11 }, {   0, 292 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   9,   0 }, {  10,   0 }, {  11,   0 }, {  12,   0 },
 {  13,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 },
 {   0,   0 }, {   0,   0 }, {   0,   0 }, {   0,   0 }, {  32,   0 },
 {   0,   9 }, {   0, 258 }, {   1,   0 }, {   2,   0 }, {   3,   0 },
 {   4,   0 }, {   5,   0 }, {   6,   0 }, {   7,   0 }, {   8,   0 },
 {   9,   0 }, {  10,   0 }, {  11,   0 }, {  12,   0 }, {  13,   0 },

 {  14,   0 }, {  15,   0 }, {  16,   0 }, {  17,   0 }, {  18,   0 },
 {  19,   0 }, {  20,   0 }, {  21,   0 }, {  22,   0 }, {  23,   0 },
 {  24,   0 }, {  25,   0 }, {  26,   0 }, {  27,   0 }, {  28,   0 },
 {  29,   0 }, {  30,   0 }, {  31,   0 }, {  32,   0 }, {  33,   0 },
 {   0,   0 }, {  35,   0 }, {  36,   0 }, {  37,   0 }, {  38,   0 },
 {  39,   0 }, {  40,   0 }, {  41,   0 }, {  42,   0 }, {  43,   0 },
 {  44,   0 }, {  45,   0 }, {  46,   0 }, {  47,   0 }, {  48,   0 },
 {  49,   0 }, {  50,   0 }, {  51,   0 }, {  52,   0 }, {  53,   0 },
 {  54,   0 }, {  55,   0 }, {  56,   0 }, {  57,   0 }, {  58,   0 },
 {  59,   0 }, {  60,   0 }, {  61,   0 }, {  62,   0 }, {  63,   0 },

 {  64,   0 }, {  65,   0 }, {  66,   0 }, {  67,   0 }, {  68,   0 },
 {  69,   0 }, {  70,   0 }, {  71,   0 }, {  72,   0 }, {  73,   0 },
 {  74,   0 }, {  75,   0 }, {  76,   0 }, {  77,   0 }, {  78,   0 },
 {  79,   0 }, {  80,   0 }, {  81,   0 }, {  82,   0 }, {  83,   0 },
 {  84,   0 }, {  85,   0 }, {  86,   0 }, {  87,   0 }, {  88,   0 },
 {  89,   0 }, {  90,   0 }, {  91,   0 }, {   0,   0 }, {  93,   0 },
 {  94,   0 }, {  95,   0 }, {  96,   0 }, {  97,   0 }, {  98,   0 },
 {  99,   0 }, { 100,   0 }, { 101,   0 }, { 102,   0 }, { 103,   0 },
 { 104,   0 }, { 105,   0 }, { 106,   0 }, { 107,   0 }, { 108,   0 },
 { 109,   0 }, { 110,   0 }, { 111,   0 }, { 112,   0 }, { 113,   0 },

 { 114,   0 }, { 115,   0 }, { 116,   0 }, { 117,   0 }, { 118,   0 },
 { 119,   0 }, { 120,   0 }, { 121,   0 }, { 122,   0 }, { 123,   0 },
 { 124,   0 }, { 125,   0 }, { 126,   0 }, { 127,   0 }, { 128,   0 },
 { 129,   0 }, { 130,   0 }, { 131,   0 }, { 132,   0 }, { 133,   0 },
 { 134,   0 }, { 135,   0 }, { 136,   0 }, { 137,   0 }, { 138,   0 },
 { 139,   0 }, { 140,   0 }, { 141,   0 }, { 142,   0 }, { 143,   0 },
 { 144,   0 }, { 145,   0 }, { 146,   0 }, { 147,   0 }, { 148,   0 },
 { 149,   0 }, { 150,   0 }, { 151,   0 }, { 152,   0 }, { 153,   0 },
 { 154,   0 }, { 155,   0 }, { 156,   0 }, { 157,   0 }, { 158,   0 },
 { 159,   0 }, { 160,   0 }, { 161,   0 }, { 162,   0 }, { 163,   0 },

 { 164,   0 }, { 165,   0 }, { 166,   0 }, { 167,   0 }, { 168,   0 },
 { 169,   0 }, { 170,   0 }, { 171,   0 }, { 172,   0 }, { 173,   0 },
 { 174,   0 }, { 175,   0 }, { 176,   0 }, { 177,   0 }, { 178,   0 },
 { 179,   0 }, { 180,   0 }, { 181,   0 }, { 182,   0 }, { 183,   0 },
 { 184,   0 }, { 185,   0 }, { 186,   0 }, { 187,   0 }, { 188,   0 },
 { 189,   0 }, { 190,   0 }, { 191,   0 }, { 192,   0 }, { 193,   0 },
 { 194,   0 }, { 195,   0 }, { 196,   0 }, { 197,   0 }, { 198,   0 },
 { 199,   0 }, { 200,   0 }, { 201,   0 }, { 202,   0 }, { 203,   0 },
 { 204,   0 }, { 205,   0 }, { 206,   0 }, { 207,   0 }, { 208,   0 },
 { 209,   0 }, { 210,   0 }, { 211,   0 }, { 212,   0 }, { 213,   0 },

 { 214,   0 }, { 215,   0 }, { 216,   0 }, { 217,   0 }, { 218,   0 },
 { 219,   0 }, { 220,   0 }, { 221,   0 }, { 222,   0 }, { 223,   0 },
 { 224,   0 }, { 225,   0 }, { 226,   0 }, { 227,   0 }, { 228,   0 },
 { 229,   0 }, { 230,   0 }, { 231,   0 }, { 232,   0 }, { 233,   0 },
 { 234,   0 }, { 235,   0 }, { 236,   0 }, { 237,   0 }, { 238,   0 },
 { 239,   0 }, { 240,   0 }, { 241,   0 }, { 242,   0 }, { 243,   0 },
 { 244,   0 }, { 245,   0 }, { 246,   0 }, { 247,   0 }, { 248,   0 },
 { 249,   0 }, { 250,   0 }, { 251,   0 }, { 252,   0 }, { 253,   0 },
 { 254,   0 }, { 255,   0 }, { 256,   0 }, { 257,  14 }, {   1,   0 },
    };

static yyconst struct yy_trans_info *yy_start_state_list[5] =
    {
    &yy_transition[1],
    &yy_transition[3],
    &yy_transition[261],
    &yy_transition[519],
    &yy_transition[777],
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#define YY_NO_UNISTD_H 1

#	include <stdio.h>
#	include <stdlib.h>
#	include <iostream>
#	include <sstream>

#	define LVAL (yylval_param)
#	define PARSER (reinterpret_cast<cli::Tokenizer *>(args_get_extra(yyscanner)))
#	define YY_INPUT(buf,result,max_size) result = PARSER->Input(buf, max_size);

static void yy_fatal_error(const char*, yyscan_t);
void (*hide_warning)(const char*, yyscan_t) = &yy_fatal_error;





#define INITIAL 0
#define STRING 1




#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif



#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif





/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;




    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;






    }; /* end struct yyguts_t */




static int yy_init_globals (yyscan_t yyscanner );




    

    


int args_lex_init (yyscan_t* scanner);

int args_lex_init_extra (YY_EXTRA_TYPE user_defined,yyscan_t* scanner);



/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */


int args_lex_destroy (yyscan_t yyscanner );



int args_get_debug (yyscan_t yyscanner );



void args_set_debug (int debug_flag ,yyscan_t yyscanner );



YY_EXTRA_TYPE args_get_extra (yyscan_t yyscanner );



void args_set_extra (YY_EXTRA_TYPE user_defined ,yyscan_t yyscanner );



FILE *args_get_in (yyscan_t yyscanner );



void args_set_in  (FILE * _in_str ,yyscan_t yyscanner );



FILE *args_get_out (yyscan_t yyscanner );



void args_set_out  (FILE * _out_str ,yyscan_t yyscanner );



			int args_get_leng (yyscan_t yyscanner );



char *args_get_text (yyscan_t yyscanner );



int args_get_lineno (yyscan_t yyscanner );



void args_set_lineno (int _line_number ,yyscan_t yyscanner );




int args_get_column  (yyscan_t yyscanner );





void args_set_column (int _column_no ,yyscan_t yyscanner );




/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int args_wrap (yyscan_t yyscanner );
#else
extern int args_wrap (yyscan_t yyscanner );
#endif
#endif


#ifndef YY_NO_UNPUT
    
#endif


#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int ,yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * ,yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT

#ifdef __cplusplus
static int yyinput (yyscan_t yyscanner );
#else
static int input (yyscan_t yyscanner );
#endif

#endif








/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif


/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, (size_t) yyleng, 1, yyout )) {} } while (0)
#endif



/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	errno=0; \
	while ( (result = (int) read( fileno(yyin), buf, (yy_size_t) max_size )) < 0 ) \
	{ \
		if( errno != EINTR) \
		{ \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
			break; \
		} \
		errno=0; \
		clearerr(yyin); \
	}\
\

#endif



/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif


/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif


/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
#endif





/* end tables serialization structures and prototypes */



/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1









extern int args_lex (yyscan_t yyscanner);

#define YY_DECL int args_lex (yyscan_t yyscanner)
#endif /* !YY_DECL */


/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif



/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif



#define YY_RULE_SETUP \
	YY_USER_ACTION



/** The main scanner function which does all the work.
 */
YY_DECL
{
	yy_state_type yy_current_state;
	char *yy_cp, *yy_bp;
	int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;







	if ( !yyg->yy_init )
		{
		yyg->yy_init = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif



		if ( ! yyg->yy_start )
			yyg->yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			args_ensure_buffer_stack (yyscanner);
			YY_CURRENT_BUFFER_LVALUE =
				args__create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
		}

		args__load_buffer_state(yyscanner );
		}

	{



	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yyg->yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yyg->yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yy_start_state_list[yyg->yy_start];
yy_match:
		{
		yyconst struct yy_trans_info *yy_trans_info;

		YY_CHAR yy_c;

		for ( yy_c = YY_SC_TO_UI(*yy_cp);
		      (yy_trans_info = &yy_current_state[(unsigned int) yy_c])->
		yy_verify == yy_c;
		      yy_c = YY_SC_TO_UI(*++yy_cp) )
			yy_current_state += yy_trans_info->yy_nxt;
		}

yy_find_action:
		yy_act = yy_current_state[-1].yy_nxt;

		YY_DO_BEFORE_ACTION;



do_action:	/* This label is used only to access EOF actions. */


		switch ( yy_act )
	{ /* beginning of action switch */
case 1:
YY_RULE_SETUP
{ BEGIN(STRING); }
	YY_BREAK
case 2:
YY_RULE_SETUP
{ PARSER->Write('\n'); }
	YY_BREAK
case 3:
YY_RULE_SETUP
{ PARSER->Write('\t'); }
	YY_BREAK
case 4:
YY_RULE_SETUP
{ PARSER->Write('\f'); }
	YY_BREAK
case 5:
YY_RULE_SETUP
{ PARSER->Write('\r'); }
	YY_BREAK
case 6:
YY_RULE_SETUP
{ PARSER->Write('\b'); }
	YY_BREAK
case 7:
YY_RULE_SETUP
{ PARSER->Write(yytext[1]); }
	YY_BREAK
case 8:
YY_RULE_SETUP
{ BEGIN(INITIAL); }
	YY_BREAK
case 9:
/* rule 9 can match eol */
YY_RULE_SETUP
{ PARSER->Write(yytext, yyleng); }
	YY_BREAK
case 10:
YY_RULE_SETUP
{ PARSER->Write(' '); }
	YY_BREAK
case 11:
/* rule 11 can match eol */
YY_RULE_SETUP
{ PARSER->NextArgument(); }
	YY_BREAK
case 12:
YY_RULE_SETUP
{ PARSER->Write(yytext[0]); }
	YY_BREAK
case 13:
YY_RULE_SETUP
ECHO;
	YY_BREAK
			case YY_STATE_EOF(INITIAL):
			case YY_STATE_EOF(STRING):
				yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yyg->yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * args_lex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state( yyscanner );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

			yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yyg->yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yyg->yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer( yyscanner ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yyg->yy_did_buffer_switch_on_eof = 0;

				if ( args_wrap(yyscanner ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yyg->yy_c_buf_p =
					yyg->yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yyg->yy_c_buf_p =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of args_lex */






/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	char *source = yyg->yytext_ptr;
	yy_size_t number_to_move, i;
	int ret_val;

	if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (yy_size_t) (yyg->yy_c_buf_p - yyg->yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER_LVALUE;

			int yy_c_buf_p_offset =
				(int) (yyg->yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					args_realloc((void *) b->yy_ch_buf,(yy_size_t) (b->yy_buf_size + 2) ,yyscanner );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = NULL;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			yyg->yy_n_chars, num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	if ( yyg->yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			args_restart(yyin  ,yyscanner);
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((int) (yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) args_realloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,(yy_size_t) new_size ,yyscanner );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	yyg->yy_n_chars += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}


/* yy_get_previous_state - get the state just before the EOB char was reached */


    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
{
	yy_state_type yy_current_state;
	char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	yy_current_state = yy_start_state_list[yyg->yy_start];

	for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
		{
		yy_current_state += yy_current_state[(*yy_cp ? YY_SC_TO_UI(*yy_cp) : 256)].yy_nxt;
		}

	return yy_current_state;
}


/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
{
	int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */

	int yy_c = 256;
	yyconst struct yy_trans_info *yy_trans_info;

	yy_trans_info = &yy_current_state[(unsigned int) yy_c];
	yy_current_state += yy_trans_info->yy_nxt;
	yy_is_jam = (yy_trans_info->yy_verify != yy_c);

	(void)yyg;
	return yy_is_jam ? 0 : yy_current_state;
}


#ifndef YY_NO_UNPUT

#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

{
	int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	*yyg->yy_c_buf_p = yyg->yy_hold_char;

	if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			/* This was really a NUL. */
			*yyg->yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = yyg->yy_c_buf_p - yyg->yytext_ptr;
			++yyg->yy_c_buf_p;

			switch ( yy_get_next_buffer( yyscanner ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					args_restart(yyin ,yyscanner);

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( args_wrap(yyscanner ) )
						return 0;

					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput(yyscanner);
#else
					return input(yyscanner);
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yyg->yy_c_buf_p;	/* cast for 8-bit char's */
	*yyg->yy_c_buf_p = '\0';	/* preserve yytext */
	yyg->yy_hold_char = *++yyg->yy_c_buf_p;


	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void args_restart  (FILE * input_file , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! YY_CURRENT_BUFFER ){
        args_ensure_buffer_stack (yyscanner);
		YY_CURRENT_BUFFER_LVALUE =
            args__create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
	}

	args__init_buffer(YY_CURRENT_BUFFER,input_file ,yyscanner);
	args__load_buffer_state(yyscanner );
}


/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
    void args__switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	/* TODO. We should be able to replace this entire function body
	 * with
	 *		args_pop_buffer_state();
	 *		args_push_buffer_state(new_buffer);
     */
	args_ensure_buffer_stack (yyscanner);
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	args__load_buffer_state(yyscanner );

	/* We don't actually know whether we did this switch during
	 * EOF (args_wrap()) processing, but the only time this flag
	 * is looked at is after args_wrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yyg->yy_did_buffer_switch_on_eof = 1;
}


static void args__load_buffer_state  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE args__create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) args_alloc(sizeof( struct yy_buffer_state ) ,yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in args__create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) args_alloc((yy_size_t) (b->yy_buf_size + 2) ,yyscanner );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in args__create_buffer()" );

	b->yy_is_our_buffer = 1;

	args__init_buffer(b,file ,yyscanner);

	return b;
}


/** Destroy the buffer.
 * @param b a buffer created with args__create_buffer()
 * @param yyscanner The scanner object.
 */
    void args__delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		args_free((void *) b->yy_ch_buf ,yyscanner );

	args_free((void *) b ,yyscanner );
}


/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a args_restart() or at EOF.
 */
    static void args__init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)

{
	int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	args__flush_buffer(b ,yyscanner);

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then args__init_buffer was _probably_
     * called from args_restart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }


    
        b->yy_is_interactive = 0;
    

	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
    void args__flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		args__load_buffer_state(yyscanner );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
void args_push_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (new_buffer == NULL)
		return;

	args_ensure_buffer_stack(yyscanner);

	/* This block is copied from args__switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		yyg->yy_buffer_stack_top++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from args__switch_to_buffer. */
	args__load_buffer_state(yyscanner );
	yyg->yy_did_buffer_switch_on_eof = 1;
}


/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
void args_pop_buffer_state (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (!YY_CURRENT_BUFFER)
		return;

	args__delete_buffer(YY_CURRENT_BUFFER ,yyscanner);
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if (yyg->yy_buffer_stack_top > 0)
		--yyg->yy_buffer_stack_top;

	if (YY_CURRENT_BUFFER) {
		args__load_buffer_state(yyscanner );
		yyg->yy_did_buffer_switch_on_eof = 1;
	}
}


/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void args_ensure_buffer_stack (yyscan_t yyscanner)
{
	int num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if (!yyg->yy_buffer_stack) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		yyg->yy_buffer_stack = (struct yy_buffer_state**)args_alloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in args_ensure_buffer_stack()" );
								  
		
		memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		yyg->yy_buffer_stack_max = num_to_alloc;
		yyg->yy_buffer_stack_top = 0;
		return;
	}

	if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

		/* Increase the buffer to prepare for a possible push. */
		yy_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
		yyg->yy_buffer_stack = (struct yy_buffer_state**)args_realloc
								(yyg->yy_buffer_stack,
								num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in args_ensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
		yyg->yy_buffer_stack_max = num_to_alloc;
	}
}





/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE args__scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) args_alloc(sizeof( struct yy_buffer_state ) ,yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in args__scan_buffer()" );

	b->yy_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = NULL;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	args__switch_to_buffer(b ,yyscanner );

	return b;
}




/** Setup the input buffer state to scan a string. The next call to args_lex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       args__scan_bytes() instead.
 */
YY_BUFFER_STATE args__scan_string (yyconst char * yystr , yyscan_t yyscanner)
{
    
	return args__scan_bytes(yystr,(int) strlen(yystr) ,yyscanner);
}




/** Setup the input buffer state to scan the given bytes. The next call to args_lex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE args__scan_bytes  (yyconst char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	yy_size_t i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (yy_size_t) _yybytes_len + 2;
	buf = (char *) args_alloc(n ,yyscanner );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in args__scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = args__scan_buffer(buf,n ,yyscanner);
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in args__scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}











#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yynoreturn yy_fatal_error (yyconst char* msg , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        yy_size_t yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = yyg->yy_hold_char; \
		yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
		yyg->yy_hold_char = *yyg->yy_c_buf_p; \
		*yyg->yy_c_buf_p = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )



/* Accessor  methods (get/set functions) to struct members. */


/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE args_get_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}



/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int args_get_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    
    
        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yylineno;
}




/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int args_get_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    
    
        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yycolumn;
}




/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *args_get_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}



/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *args_get_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}



/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int args_get_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}


/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *args_get_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}



/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void args_set_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}



/** Set the current line number.
 * @param _line_number line number
 * @param yyscanner The scanner object.
 */
void args_set_lineno (int  _line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    
        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "args_set_lineno called with no buffer" );
    
    yylineno = _line_number;
}




/** Set the current column.
 * @param _column_no column number
 * @param yyscanner The scanner object.
 */
void args_set_column (int  _column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    
        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "args_set_column called with no buffer" );
    
    yycolumn = _column_no;
}





/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see args__switch_to_buffer
 */
void args_set_in (FILE *  _in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = _in_str ;
}



void args_set_out (FILE *  _out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = _out_str ;
}




int args_get_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}



void args_set_debug (int  _bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = _bdebug ;
}


/* Accessor methods for yylval and yylloc */



/* User-visible API */

/* args_lex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */

int args_lex_init(yyscan_t* ptr_yy_globals)

{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) args_alloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}


/* args_lex_init_extra has the same functionality as args_lex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to args_alloc in
 * the yyextra field.
 */

int args_lex_init_extra(YY_EXTRA_TYPE yy_user_defined,yyscan_t* ptr_yy_globals )

{
    struct yyguts_t dummy_yyguts;

    args_set_extra (yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }
	
    *ptr_yy_globals = (yyscan_t) args_alloc ( sizeof( struct yyguts_t ), &dummy_yyguts );
	
    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }
    
    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));
    
    args_set_extra (yy_user_defined, *ptr_yy_globals);
    
    return yy_init_globals ( *ptr_yy_globals );
}


static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from args_lex_destroy(), so don't allocate here.
     */


    yyg->yy_buffer_stack = NULL;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = NULL;
    yyg->yy_init = 0;
    yyg->yy_start = 0;


    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;






/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = NULL;
    yyout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * args_lex_init()
     */
    return 0;
}


/* args_lex_destroy is for both reentrant and non-reentrant scanners. */
int args_lex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		args__delete_buffer(YY_CURRENT_BUFFER ,yyscanner );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		args_pop_buffer_state(yyscanner);
	}

	/* Destroy the stack itself. */
	args_free(yyg->yy_buffer_stack ,yyscanner);
	yyg->yy_buffer_stack = NULL;


    /* Destroy the start condition stack. */
        args_free(yyg->yy_start_stack ,yyscanner );
        yyg->yy_start_stack = NULL;




    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * args_lex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

    /* Destroy the main struct (reentrant only). */
    args_free ( yyscanner , yyscanner );
    yyscanner = NULL;
    return 0;
}



/*
 * Internal utility routines.
 */



#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif



#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s , yyscan_t yyscanner)
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif



void *args_alloc (yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	return malloc(size);
}



void *args_realloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}



void args_free (void * ptr , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	free( (char *) ptr );	/* see args_realloc() for (char *) cast */
}


#define YYTABLES_NAME "yytables"









